package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.bean.ScannerLoginInfo;
import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.entity.SystemConfigEntity;
import cn.com.qmth.scancentral.entity.UserEntity;
import cn.com.qmth.scancentral.enums.Role;
import cn.com.qmth.scancentral.exception.ForbiddenExceptions;
import cn.com.qmth.scancentral.exception.UnauthorizedExceptions;
import cn.com.qmth.scancentral.service.AuthService;
import cn.com.qmth.scancentral.service.ScannerService;
import cn.com.qmth.scancentral.service.SessionService;
import cn.com.qmth.scancentral.service.UserService;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.security.annotation.AuthorizationComponent;
import com.qmth.boot.core.security.service.AuthorizationService;
import com.qmth.boot.tools.signature.SignatureType;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
@AuthorizationComponent
public class AuthServiceImpl implements AuthorizationService<User>, AuthService {

    private static final Logger log = LoggerFactory.getLogger(AuthServiceImpl.class);

    @Autowired
    private UserService userService;

    @Autowired
    private SessionService sessionService;

    @Autowired
    private ScannerService scannerService;

    @Autowired
    private SystemConfigServiceImpl systemConfigService;

    @Override
    public User findByIdentity(String identity, SignatureType type, String path) {
        return sessionService.getSessionUser(identity);
    }

    @Override
    public boolean hasPermission(User user, String path) {
        sessionService.updateUserSession(user);
        return true;
    }

    @Override
    public User login(String loginName, String password, String deviceIp) {
        UserEntity userEntity = userService.getByLoginName(loginName);
        if (userEntity == null) {
            throw new ParameterException("用户不存在");
        }
        if (!userEntity.getEnable()) {
            throw new ParameterException("用户已禁用");
        }
        if (!password.equals(userEntity.getPassword())) {
            throw new ParameterException("密码错误");
        }

        LambdaUpdateWrapper<UserEntity> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(UserEntity::getDevice, deviceIp);
        updateWrapper.eq(UserEntity::getId, userEntity.getId());
        userService.update(updateWrapper);

        User user = new User();
        user.setRole(userEntity.getRole());
        user.setAccount(loginName);
        user.setAccessToken(RandomStringUtils.randomAlphanumeric(32));
        user.buildKey();
        user.setName(userEntity.getName());
        user.setId(userEntity.getId());
        sessionService.userLogin(user);
        log.warn("登录成功! role:{} loginName:{}", userEntity.getRole(), loginName);
        return user;
    }

    @Transactional
    @Override
    public User scannerLogin(ScannerLoginInfo loginInfo) {
        if (StringUtils.isBlank(loginInfo.getDevice())) {
            throw new ParameterException("device不能为空");
        }
        if (StringUtils.isBlank(loginInfo.getDeviceName())) {
            throw new ParameterException("deviceName不能为空");
        }

        SystemConfigEntity config = systemConfigService.find();
        if (config != null) {
            if (config.getScannerEnableLogin() != null && !config.getScannerEnableLogin()) {
                throw ForbiddenExceptions.SCANNER_LOGIN_DISABLE;
            }
            if (StringUtils.isNotEmpty(config.getScannerPassword())) {
                if (loginInfo.getPassword() == null) {
                    throw UnauthorizedExceptions.SCANNER_PASSWORD_INVALID;
                }
                if (!config.getScannerPassword().equals(loginInfo.getPassword())) {
                    throw UnauthorizedExceptions.SCANNER_PASSWORD_INVALID;
                }
            }
        }

        User user = new User();
        user.setRole(Role.SCANNER);
        user.setAccount(loginInfo.getDevice());
        user.setAccessToken(RandomStringUtils.randomAlphanumeric(32));
        user.buildKey();
        user.setName(loginInfo.getDeviceName());
        scannerService.onLogin(loginInfo);
        sessionService.userLogin(user);

        log.warn("登录成功! role:{} device:{}_{} {}", Role.SCANNER, loginInfo.getDevice(), loginInfo.getDeviceName(), loginInfo.getDeviceIp());
        return user;
    }

}
