package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.bean.ImportStudentDomain;
import cn.com.qmth.scancentral.enums.AsyncTaskStatus;
import cn.com.qmth.scancentral.service.StudentImportService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.service.SubjectService;
import cn.com.qmth.scancentral.vo.studentimport.ImportTaskVo;
import cn.com.qmth.scancentral.vo.studentimport.StudentImportInfo;
import com.qmth.boot.core.exception.ParameterException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;

@Component
public class StudentImportServiceImpl implements StudentImportService {

    private static final Logger log = LoggerFactory.getLogger(StudentImportServiceImpl.class);

    private final static Map<String, ImportTaskVo> IMPORT_TASKS = new ConcurrentHashMap<>();

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private StudentService studentService;

    @Override
    public ImportTaskVo studentImportProgress(String taskId) {
        ImportTaskVo task = IMPORT_TASKS.get(taskId);
        if (task == null) {
            throw new ParameterException("考生导入任务不存在");
        }
        return task;
    }

    @Override
    public boolean existRunningStudentImportTask() {
        for (ImportTaskVo task : IMPORT_TASKS.values()) {
            if (AsyncTaskStatus.RUNNING == task.getStatus()) {
                return true;
            }
        }
        return false;
    }

    @Async
    @Override
    public void studentImport(String taskId, List<String> lines, Long examId, String subjectCode, Integer yearConfig,
                              Integer yearHalfConfig) {
        ImportTaskVo task = new ImportTaskVo();
        task.setTaskId(taskId);
        task.setProgress(0d);
        task.setStatus(AsyncTaskStatus.RUNNING);
        IMPORT_TASKS.put(taskId, task);

        List<StudentImportInfo> list = new ArrayList<>();
        try {
            this.parseValues(lines, list, examId, subjectCode, yearConfig, yearHalfConfig);
            log.warn("【考生导入】解析考生数据，有效共{}条! taskId:{}", list.size(), taskId);
        } catch (Exception e) {
            log.error("【考生导入】解析考生数据错误! taskId:{} {}", taskId, e.getMessage());
            task.setStatus(AsyncTaskStatus.FAILED);
            task.setErrMsg(e.getMessage());
            return;
        }

        int total = list.size(), finishCount = 0;
        try {
            log.info("【考生导入】执行导入数据库开始... {}", taskId);
            long startTime = System.currentTimeMillis();

            int batchNum = 1000;
            List<ImportStudentDomain> batchList = new ArrayList<>();

            for (int n = 0; n < total; n++) {
                batchList.add(ofData(list.get(n)));

                if ((n + 1) % batchNum == 0) {
                    studentService.importStudent(batchList);
                    finishCount = finishCount + batchList.size();
                    batchList.clear();

                    double progress = finishCount * 100f / total;
                    long cost = (System.currentTimeMillis() - startTime) / 1000L;
                    log.info("【考生导入】共{}条 已导入:{}条 已耗时:{}秒 进度:{}%", total, finishCount, cost, progress);
                    task.setProgress(progress);
                }
            }

            if (!batchList.isEmpty()) {
                studentService.importStudent(batchList);
                finishCount = finishCount + batchList.size();
            }

            task.setStatus(AsyncTaskStatus.SUCCESS);
            task.setProgress(100d);
            long cost = (System.currentTimeMillis() - startTime) / 1000L;
            log.info("【考生导入】共{}条 已导入:{}条 已耗时:{}秒 进度:100% taskId:{}", total, finishCount, cost, taskId);
        } catch (Exception e) {
            task.setStatus(AsyncTaskStatus.FAILED);
            task.setErrMsg(String.format("导入异常终止，已导入%s条，未导入%s条！", finishCount, total - finishCount));
            log.error("【考生导入】导入异常终止，已导入{}条，未导入{}条！taskId:{} err:{}", finishCount, total - finishCount, taskId,
                    e.getMessage(), e);
        }
    }

    private ImportStudentDomain ofData(StudentImportInfo info) {
        ImportStudentDomain data = new ImportStudentDomain();
        data.setExamId(info.getExamId());
        data.setSubjectCode(info.getSubjectCode());
        data.setExamNumber(info.getExamNumber());
        data.setName(info.getName());
        data.setCampusCode(info.getCampusCode());
        data.setCampusName(info.getCampusName());
        data.setExamSite(info.getExamSite());
        data.setExamSiteName(info.getExamSiteName());
        data.setExamRoom(info.getExamRoom());
        data.setPackageCode(info.getPackageCode());
        data.setSeatNumber(info.getSeatNumber());
        data.setProvince(info.getProvince());
        return data;
    }

    private void parseValues(List<String> lines, List<StudentImportInfo> list, Long examId, String subjectCodeConfig,
                             Integer yearConfig, Integer yearHalfConfig) {
        Set<String> subjectCodes = new HashSet<>();
        Set<String> examNumbers = new HashSet<>();
        Pattern examNumberPattern = Pattern.compile("^[0-9]{15}$");

        for (int n = 1; n < lines.size(); n++) {
            String line = lines.get(n);
            if (StringUtils.isBlank(line)) {
                continue;
            }

            line = StringUtils.deleteWhitespace(line);
            String[] values = StringUtils.splitByWholeSeparatorPreserveAllTokens(line, ",");
            if (values.length != 3) {
                throw new RuntimeException(String.format("【第%s行】数据格式不正确", n + 1));
            }

            // 默认模板：准考证号,姓名,考点名称（标题行，数据从第二行读取）
            String examNumber = values[0].trim();
            String name = values[1].trim();
            String examSiteName = values[2].trim();

            if (StringUtils.isEmpty(examNumber)) {
                throw new RuntimeException(String.format("【第%s行】准考证号不能为空", n + 1));
            }
            if (StringUtils.isEmpty(name)) {
                throw new RuntimeException(String.format("【第%s行】姓名不能为空", n + 1));
            }
            if (StringUtils.isEmpty(examSiteName)) {
                throw new RuntimeException(String.format("【第%s行】考点名称不能为空", n + 1));
            }
            if (examNumbers.contains(examNumber)) {
                throw new RuntimeException(String.format("【第%s行】考生信息存在重复", n + 1));
            }
            examNumbers.add(examNumber);

            if (!examNumberPattern.matcher(examNumber).find()) {
                throw new RuntimeException(String.format("【第%s行】准考证号不是15位数字", n + 1));
            }

            String year = examNumber.substring(6, 8);
            if (!yearConfig.toString().equals(year)) {
                throw new RuntimeException(String.format("【第%s行】考试年度不正确", n + 1));
            }

            String yearHalf = examNumber.substring(8, 9);
            if (!yearHalfConfig.toString().equals(yearHalf)) {
                throw new RuntimeException(String.format("【第%s行】考次不正确", n + 1));
            }

            String subjectCode = examNumber.substring(9, 10);
            if (StringUtils.isNotBlank(subjectCodeConfig) && !subjectCodeConfig.equals(subjectCode)) {
                throw new RuntimeException(String.format("【第%s行】科目代码不正确", n + 1));
            }

            String province = examNumber.substring(0, 3);
            String examSite = examNumber.substring(0, 5);
            String campusCode = examNumber.substring(5, 6);
            String examRoom = examNumber.substring(10, 13);
            String seatNumber = examNumber.substring(13, 15);
            subjectCodes.add(subjectCode);

            StudentImportInfo data = new StudentImportInfo();
            data.setProvince(province);
            data.setExamId(examId);
            data.setSubjectCode(subjectCode);
            data.setExamNumber(examNumber);
            data.setStudentCode(examNumber);
            data.setName(name);
            data.setExamSite(examSite);
            data.setExamSiteName(examSiteName);
            data.setCampusCode(campusCode);
            data.setCampusName(campusCode);
            data.setExamRoom(examRoom);
            data.setPackageCode(examSite + campusCode + examRoom);
            data.setSeatNumber(seatNumber);
            list.add(data);
        }

        // 校验所有科目代码
        for (String subjectCode : subjectCodes) {
            if (subjectService.findByExamIdAndCode(examId, subjectCode) == null) {
                log.warn("科目不存在！examId:{} subjectCode:{}", examId, subjectCode);
                throw new RuntimeException(String.format("科目代码【%s】不存在！", subjectCode));
            }
        }
    }

}
