package cn.com.qmth.scancentral.util;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.io.Files;
import com.qmth.boot.core.exception.StatusException;

public class FileUtil {

    private static final Logger log = LoggerFactory.getLogger(FileUtil.class);

    /**
     * 创建文件目录
     */
    public static void makeDirs(String dirPath) {
        if (StringUtils.isBlank(dirPath)) {
            return;
        }
        File dir = new File(dirPath);
        if (!dir.exists()) {
            dir.mkdirs();
        }
    }

    /**
     * 获取文件后缀名，包括"."
     */
    public static String getFileSuffix(String fileName) {
        if (StringUtils.isNotEmpty(fileName)) {
            int index = fileName.lastIndexOf(".");
            if (index > 0) {
                return fileName.substring(index).toLowerCase();
            }
        }

        return "";
    }

    /**
     * 读取文件内容（一次性读取）
     */
    public static List<String> readAllLines(File file) {
        try {
            return Files.readLines(file, StandardCharsets.UTF_8);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RuntimeException("读取文件失败！");
        }
    }

    public static String readFileContent(InputStream in) {
        StringBuilder content = new StringBuilder();
        InputStreamReader streamReader = null;
        BufferedReader bufferedReader = null;
        try {
            String encoding = "UTF-8";
            streamReader = new InputStreamReader(in, encoding);
            bufferedReader = new BufferedReader(streamReader);
            String line;
            while ((line = bufferedReader.readLine()) != null) {
                content.append(line);
            }
            return content.toString();
        } catch (IOException e) {
            throw new StatusException("出错", e);
        } finally {
            if (streamReader != null) {
                try {
                    streamReader.close();
                } catch (IOException e) {
                }
            }
            if (bufferedReader != null) {
                try {
                    bufferedReader.close();
                } catch (IOException e) {
                }
            }
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    /**
     * 获得文件的byte数组
     *
     * @param filePath
     * @return
     * @throws IOException
     */
    public static byte[] getBytes(String filePath) throws IOException {
        File file = new File(filePath);
        if (!file.exists()) {
            throw new FileNotFoundException(filePath);
        }

        ByteArrayOutputStream bos = new ByteArrayOutputStream(((int) file.length()));
        BufferedInputStream in = null;
        try {
            in = new BufferedInputStream(new FileInputStream(file));
            int bufSize = 1024;
            byte[] buffer = new byte[bufSize];
            int len = 0;
            while (-1 != (len = in.read(buffer, 0, bufSize))) {
                bos.write(buffer, 0, len);
            }
            return bos.toByteArray();
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            bos.close();
        }
    }

    public static void dozip(File target, File[] files) {
        // 压缩后的 zip 文件名
        if (target == null || !target.exists()) {
            throw new RuntimeException("目录或文件不能为空！");
        }
        ZipOutputStream zipOut = null;
        try {
            // 创建 ZipOutputStream 对象
            FileOutputStream fos = new FileOutputStream(target);
            zipOut = new ZipOutputStream(fos, Charset.forName("UTF-8"));

            // 循环每个文件并将其添加到压缩包
            for (File vo : files) {
                FileInputStream in = new FileInputStream(vo);
                try {
                    ZipEntry zipEntry = new ZipEntry(vo.getName());
                    zipOut.putNextEntry(zipEntry);
                    // 将文件内容写入 ZipOutputStream
                    byte[] bytes = new byte[1024];
                    int length;
                    while ((length = in.read(bytes)) >= 0) {
                        zipOut.write(bytes, 0, length);
                    }
                } finally {
                    // 关闭当前文件的输入流
                    in.close();
                }
            }

        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            try {
                if (zipOut != null) {
                    zipOut.close();
                }
            } catch (IOException e) {
            }
        }
    }

    public static void deleteDirectory(File dirFile) {
        if (!dirFile.exists()) {
            return;
        }
        if (dirFile.isFile()) {
            dirFile.delete();
        } else {
            File[] files = dirFile.listFiles();
            if (files != null) {
                for (int i = 0; i < files.length; i++) {
                    deleteDirectory(files[i]);
                }
            }
            dirFile.delete();
        }
    }
}
